/*
 * Copyright (c) 2016 Intel Corporation
 *
 * SPDX-License-Identifier: Apache-2.0
 */

#include <stdio.h>
#include <zephyr/kernel.h>
#include <zephyr/drivers/gpio.h>
#include <zephyr/drivers/uart.h>
#include <string.h>
#include <zephyr/device.h>
#include <string.h>


//add from RPI
#include <stdlib.h>
#include <stdbool.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>


/* 1000 msec = 1 sec */
#define SLEEP_TIME_MS   1000

/* The devicetree node identifier for the "led0" alias. */
#define LED0_NODE DT_ALIAS(led0)

/*
 * A build error on this line means your board is unsupported.
 * See the sample documentation for information on how to fix this.
 */
static const struct gpio_dt_spec led = GPIO_DT_SPEC_GET(LED0_NODE, gpios);

/* UART Ports configuration */

const struct device *uart0 = DEVICE_DT_GET(DT_NODELABEL(uart0));
const struct device *uart1 = DEVICE_DT_GET(DT_NODELABEL(uart1));

struct uart_config uart_cfg = {
	.baudrate = 9600,
	.parity = UART_CFG_PARITY_NONE,
	.stop_bits = UART_CFG_STOP_BITS_1,
	.flow_ctrl = UART_CFG_FLOW_CTRL_NONE,
	.data_bits = UART_CFG_DATA_BITS_8,
};

void send_str(const struct device *uart, char *str, int msg_len)
{

	for (int i = 0; i < msg_len; i++) {
		uart_poll_out(uart, str[i]);
	}

//	printk("Device %s sent: \"%s\"\n", uart->name, str);
}

void recv_str(const struct device *uart, char *str, size_t i)
{
	char *head = str;
	char c;


	while (!uart_poll_in(uart, &c) && i < 3) {
		*head++ = c;
		i++;
	}
	*head = '\0';

//	printk("Device %s received: \"%s\"\n", uart->name, str);
}



int main(void)
{
	int rc;
	char txrx_buf[64];
	int i = 10;

	int ret;
	bool led_state = true;

	rc = uart_configure(uart0, &uart_cfg);
	if (rc) {
		printk("Could not configure device %s", uart0->name);
	}
	rc = uart_configure(uart1, &uart_cfg);
	if (rc) {
		printk("Could not configure device %s", uart1->name);
	}


	if (!gpio_is_ready_dt(&led)) {
		return 0;
	}

	ret = gpio_pin_configure_dt(&led, GPIO_OUTPUT_ACTIVE);
	if (ret < 0) {
		return 0;
	}

	while (1) {
		ret = gpio_pin_toggle_dt(&led);
		if (ret < 0) {
			return 0;
		}

		led_state = !led_state;
		printf("LED state: %s\n", led_state ? "ON" : "OFF");
		k_msleep(SLEEP_TIME_MS);

		txrx_buf[0] = 'A';
		send_str(uart1, txrx_buf,1);

		i = 10;
		while (i--) {
			k_sleep(K_MSEC(100));
			recv_str(uart1, txrx_buf,2);
		}
		return 0;
	}	
}